@@ -28,18 +28,18 @@
 #include "sgm.hpp"
 
 
-template<typename T> 
-CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows, int nb_cols, int nb_disps,
-    T invalid_value, bool cost_paths, bool overcounting)
+template<typename T>
+CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, unsigned long int nb_rows, unsigned long int nb_cols,
+    unsigned int nb_disps, T invalid_value, float* segmentation, bool cost_paths, bool overcounting)
 
 {
     int nb_dir = 8;
-    CostVolumes cvs;
     //Allocate final cost volume
+    CostVolumes cvs;
+    // To avoid an overflow due to big multiplications, nb_rows and nb_cols are defined as long int
     cvs.cost_volume = new float[nb_rows*nb_cols*nb_disps]();
-
     // Allocate costs
-    int nb_values=1;
+    unsigned long int nb_values=1;
     if (cost_paths){
     nb_values = nb_rows*nb_cols*nb_dir;
     }
@@ -92,6 +92,20 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
     int  pos0, pos1, pos2, pos3;
     // temporary variables for storing current minimums
     float min0, min1, min2, min3;
+    // temporary buffers for storing current classification
+    float current_class;
+    // buff_class0 stores the class from the left of current pixel (previous seen pixel)
+    float buff_class0;
+    // tmp_buff_class1_2_3 stores the previous classes of pixels seen in the line
+    float * tmp_buff_class1_2_3 = new float[nb_cols]();
+    // buff_class1_2_3 stores the previous line of classes
+    float * buff_class1_2_3 = new float[nb_cols]();
+    // reset variables store 0 if we want to reset history, 1 if not
+    float reset0;
+    float reset1;
+    float reset2;
+    float reset3;
+
 
     for (int row = 0; row < nb_rows; row++)
     {
@@ -109,6 +123,8 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
             // initialize minimums
             min0 = std::numeric_limits<float>::max() , min1 = std::numeric_limits<float>::max(), min2 = std::numeric_limits<float>::max(), min3 = std::numeric_limits<float>::max();
             pos0 = 0, pos1 = 0, pos2 = 0, pos3 = 0;
+            // Get current class of pixel
+            current_class = segmentation[col+row*nb_cols];
             for (int disp = 0; disp < nb_disps; disp++)
             {
                 costAggr = 0;
@@ -116,19 +132,23 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
                 pixelCost = cv_in[disp+col*nb_disps+row*nb_disps*nb_cols];
                 //left-> right
                 s0 = aggregatedCostFromTopLeft0(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[0].P1, penalty[0].P2, direction[0], buff0, &min_disp0, &pixel_0);
+                 nb_cols, nb_disps, penalty[0].P1, penalty[0].P2, direction[0], buff0, &min_disp0,
+                  &pixel_0, current_class, buff_class0, reset0);
                 costAggr += s0;
                 //up -> down
                 s1 = aggregatedCostFromTopLeft1(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[1].P1, penalty[1].P2, direction[1], buff1, &min_disp1, &pixel_1);
+                 nb_cols, nb_disps, penalty[1].P1, penalty[1].P2, direction[1], buff1, &min_disp1,
+                  &pixel_1, current_class, buff_class1_2_3, reset1);
                 costAggr += s1;
                 //diagonal from top left
                 s2 = aggregatedCostFromTopLeft2(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[2].P1, penalty[2].P2, direction[2], buff2, buff_disp_2, &min_disp2, &pixel_2);
+                 nb_cols, nb_disps, penalty[2].P1, penalty[2].P2, direction[2], buff2, buff_disp_2,
+                  &min_disp2, &pixel_2, current_class, buff_class1_2_3, reset2);
                 costAggr += s2;
                 //diagonal from top right
                 s3 = aggregatedCostFromTopLeft3(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[3].P1, penalty[3].P2, direction[3], buff3, &min_disp3);
+                 nb_cols, nb_disps, penalty[3].P1, penalty[3].P2, direction[3], buff3, &min_disp3,
+                  current_class, buff_class1_2_3, reset3);
                 costAggr += s3;
 
                 cvs.cost_volume[disp + col*nb_disps + row*nb_disps*nb_cols] += costAggr;
@@ -146,7 +166,12 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
                 cvs.cost_volume_min[2 + col*nb_dir + row*nb_dir*nb_cols] = pos2;
                 cvs.cost_volume_min[3 + col*nb_dir + row*nb_dir*nb_cols] = pos3;
             }
+            // Update buffer
+            buff_class0 = current_class;
+            tmp_buff_class1_2_3[col] = current_class;
         }
+        // Update buffer
+        std::copy(tmp_buff_class1_2_3, tmp_buff_class1_2_3+nb_cols, buff_class1_2_3);
     }
     //delete buffers
     delete [] buff0;
@@ -154,7 +179,9 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
     delete [] buff2;
     delete [] buff3;
     delete [] buff_disp_2;
-    
+    delete [] buff_class1_2_3;
+    delete [] tmp_buff_class1_2_3;
+
     /* ------------------*/
     /* -- Second pass -- */
     /* ------------------*/
@@ -173,6 +200,21 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
     int  pos4, pos5, pos6, pos7;
     // temporary variables for storing current minimums
     float min4, min5, min6, min7;
+
+    // temporary buffers for storing current classification
+    // buff_class4 stores the class from the right of current pixel (previous seen pixel)
+    float buff_class4;
+    // buff_class5_6_7 stores the previous line of classes
+    float * buff_class5_6_7 = new float[nb_cols]();
+    // tmp_buff_class5_6_7 stores the previous classes of pixels seen in the line
+    float * tmp_buff_class5_6_7 = new float[nb_cols]();
+    // reset variables store 0 if we want to reset history, 1 if not
+    float reset4;
+    float reset5;
+    float reset6;
+    float reset7;
+
+
     int overcounting_factor;
 
     if (overcounting)
@@ -202,6 +244,8 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
             // initialize minimums
             min4 = std::numeric_limits<float>::max(), min5 = std::numeric_limits<float>::max(), min6 = std::numeric_limits<float>::max(), min7 = std::numeric_limits<float>::max();
             pos4 = 0, pos5 = 0, pos6 = 0, pos7 = 0;
+            // Get current class of pixel
+            current_class = segmentation[col+row*nb_cols];
 
             for (int disp = 0; disp < nb_disps; disp++)
             {
@@ -211,19 +255,23 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
                 pixelCost = cv_in[disp+col*nb_disps+row*nb_disps*nb_cols];
                 //right->left
                 s4 = aggregatedCostFromBottomRight4(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[4].P1, penalty[4].P2, direction[4], buff4, &min_disp4, &pixel_4);
+                 nb_cols, nb_disps, penalty[4].P1, penalty[4].P2, direction[4], buff4, &min_disp4, &pixel_4,
+                  current_class, buff_class4, reset4);
                 costAggr += s4;
                 //up -> down
                 s5 = aggregatedCostFromBottomRight5(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[5].P1, penalty[5].P2, direction[5], buff5, &min_disp5, &pixel_5);
+                 nb_cols, nb_disps, penalty[5].P1, penalty[5].P2, direction[5], buff5, &min_disp5, &pixel_5,
+                  current_class, buff_class5_6_7, reset5);
                 costAggr += s5;
                 //diagonal from lower left
                 s6 = aggregatedCostFromBottomRight6(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[6].P1, penalty[6].P2, direction[6], buff6, buff_disp_6, &min_disp6, &pixel_6);
+                 nb_cols, nb_disps, penalty[6].P1, penalty[6].P2, direction[6], buff6, buff_disp_6, &min_disp6,
+                  &pixel_6, current_class, buff_class5_6_7, reset6);
                 costAggr += s6;
                 //diagonal from lower right
                 s7 = aggregatedCostFromBottomRight7(pixelCost, row, col, disp, invalid_value, nb_rows,
-                 nb_cols, nb_disps, penalty[7].P1, penalty[7].P2, direction[7], buff7, &min_disp7);
+                 nb_cols, nb_disps, penalty[7].P1, penalty[7].P2, direction[7], buff7, &min_disp7, current_class,
+                  buff_class5_6_7, reset7);
                 costAggr += s7;
 
                 cvs.cost_volume[disp + col*nb_disps + row*nb_disps*nb_cols] += costAggr;
@@ -245,7 +293,13 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
                 cvs.cost_volume_min[7 + col*nb_dir + row*nb_dir*nb_cols] = pos7;
             }
 
+            // Update buffer
+            buff_class4 = current_class;
+            tmp_buff_class5_6_7[col] = current_class;
+
         }
+        // Update buffer
+        std::copy(tmp_buff_class5_6_7, tmp_buff_class5_6_7+nb_cols, buff_class5_6_7);
     }
 
     delete [] buff4;
@@ -253,6 +307,9 @@ CostVolumes sgm(T * cv_in, T* p1_in, T* p2_in, int* directions_in, int nb_rows,
     delete [] buff6;
     delete [] buff7;
     delete [] buff_disp_6;
+    delete [] buff_class5_6_7;
+    delete [] tmp_buff_class5_6_7;
+
 
     return cvs;
 }
@@ -267,10 +324,10 @@ std::pair<float, int> update_minimum(float current_min, float value, int current
     }
 }
 
-template<typename T> 
+template<typename T>
 T aggregatedCostFromTopLeft0(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff0, T * min_disp0, T * pixel_0)
+    T * buff0, T * min_disp0, T * pixel_0, float current_class, float buff_class0, float &reset0)
 {
 
     //Pixel cost at the point (row,col,disp)
@@ -284,9 +341,12 @@ T aggregatedCostFromTopLeft0(T pixelCost, int row, int col, int disp, T invalid_
     {   /*Minimum cost at previous point for each disparity
         Compute value just at the first disparity for a given point (row,col)
         */
+
         if (disp == 0)
         {
             *min_disp0 = *std::min_element(&buff0[disp], &buff0[nb_disps]);
+            // if classes are different, reset history (reset == 0)
+            reset0 = current_class == buff_class0;
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -303,7 +363,7 @@ T aggregatedCostFromTopLeft0(T pixelCost, int row, int col, int disp, T invalid_
             const T tmp4 = (*min_disp0)+P2;
 
             //Minimum path cost
-            costAggr0 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp0);
+            costAggr0 += reset0 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp0));
         }
     }
     //sava buffer data before writing new value in buffer
@@ -314,10 +374,10 @@ T aggregatedCostFromTopLeft0(T pixelCost, int row, int col, int disp, T invalid_
     return costAggr0;
 }
 
-template<typename T> 
+template<typename T>
 T aggregatedCostFromTopLeft1(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff1, T * min_disp1, T * pixel_1)
+    T * buff1, T * min_disp1, T * pixel_1, float current_class, float * buff_class1, float & reset1)
 {
     /* ------------------*/
     /* --  Up->Down   -- */
@@ -332,6 +392,8 @@ T aggregatedCostFromTopLeft1(T pixelCost, int row, int col, int disp, T invalid_
         if (disp == 0)
         {
             *min_disp1 = *std::min_element(&buff1[disp+col*nb_disps], &buff1[nb_disps + col*nb_disps]);
+            // if classes are different, reset history
+            reset1 = current_class == buff_class1[col-direction.dcol];
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -348,7 +410,7 @@ T aggregatedCostFromTopLeft1(T pixelCost, int row, int col, int disp, T invalid_
             //Minimum cost at previous point
             const T tmp4 = (*min_disp1)+P2;
             //Minimum path cost
-            costAggr1 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp1);
+            costAggr1 += reset1 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp1));
         }
     }
     //sava buffer data before writing new value in buffer
@@ -360,16 +422,16 @@ T aggregatedCostFromTopLeft1(T pixelCost, int row, int col, int disp, T invalid_
 
 }
 
-template<typename T> 
+template<typename T>
 T aggregatedCostFromTopLeft2(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff2, T * buff_disp_2, T * min_disp2, T * pixel_2)
+    T * buff2, T * buff_disp_2, T * min_disp2, T * pixel_2, float current_class, float * buff_class2, float & reset2)
 {
 
     /* -----------------------------*/
     /* --Diagonal from upper left-- */
     /* -----------------------------*/
-    uint8_t costAggr2 = pixelCost;
+    T costAggr2 = pixelCost;
     //Border check
     if (! (row-direction.drow<0 || row-direction.drow > (nb_rows -1) || col-direction.dcol < 0 || col-direction.dcol > (nb_cols-1) ) )
     {
@@ -379,6 +441,8 @@ T aggregatedCostFromTopLeft2(T pixelCost, int row, int col, int disp, T invalid_
         if (disp == 0)
         {
             *min_disp2 = *std::min_element(&buff_disp_2[disp], &buff_disp_2[nb_disps]);
+            // if classes are different, reset history
+            reset2 = current_class == buff_class2[col-direction.dcol];
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -394,7 +458,7 @@ T aggregatedCostFromTopLeft2(T pixelCost, int row, int col, int disp, T invalid_
             //Minimum cost at previous point
             const T tmp4 = (*min_disp2)+P2;
             //Minimum path cost
-            costAggr2 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp2);
+            costAggr2 += reset2 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp2));
         }
     }
     //sava buffer data before writing new value in buffer_disp
@@ -407,12 +471,14 @@ T aggregatedCostFromTopLeft2(T pixelCost, int row, int col, int disp, T invalid_
     return costAggr2;
 }
 
-template<typename T> 
+template<typename T>
 T aggregatedCostFromTopLeft3(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff3, T * min_disp3)
+    T * buff3, T * min_disp3, float current_class, float * buff_class3, float & reset3)
 {
-
+    /* -----------------------------*/
+    /* --Diagonal from upper right-- */
+    /* -----------------------------*/
     T costAggr3 = pixelCost;
     //Border check
     if (! (row-direction.drow<0 || row-direction.drow > (nb_rows -1) || col-direction.dcol < 0 || col-direction.dcol > (nb_cols-1) ) )
@@ -423,6 +489,8 @@ T aggregatedCostFromTopLeft3(T pixelCost, int row, int col, int disp, T invalid_
         if (disp == 0)
         {
             *min_disp3 = *std::min_element(&buff3[disp+(col-direction.dcol)*nb_disps], &buff3[nb_disps + (col-direction.dcol)*nb_disps]);
+            // if classes are different, reset history
+            reset3 = current_class == buff_class3[col-direction.dcol];
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -430,15 +498,15 @@ T aggregatedCostFromTopLeft3(T pixelCost, int row, int col, int disp, T invalid_
         if(pixelCost != invalid_value)
         {
             //Previous cost
-            const uint8_t tmp1 = buff3[disp+(col-direction.dcol)*nb_disps];
+            const T tmp1 = buff3[disp+(col-direction.dcol)*nb_disps];
             //Previous cost at disparity-1
-            const uint8_t tmp2 = (disp>0) ? buff3[disp-1+(col-direction.dcol)*nb_disps]+P1 : std::numeric_limits<T>::max();
+            const T tmp2 = (disp>0) ? buff3[disp-1+(col-direction.dcol)*nb_disps]+P1 : std::numeric_limits<T>::max();
             //Previous cost at disparity+1
-            const uint8_t tmp3 = (disp<nb_disps-1) ? buff3[disp+1+(col-direction.dcol)*nb_disps]+P1 : std::numeric_limits<T>::max();
+            const T tmp3 = (disp<nb_disps-1) ? buff3[disp+1+(col-direction.dcol)*nb_disps]+P1 : std::numeric_limits<T>::max();
             //Minimum cost at previous point
-            const uint8_t tmp4 = (*min_disp3)+P2;
+            const T tmp4 = (*min_disp3)+P2;
             //Minimum path cost
-            costAggr3 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp3);
+            costAggr3 += reset3 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp3));
         }
     }
     //Save data in buffer
@@ -447,10 +515,10 @@ T aggregatedCostFromTopLeft3(T pixelCost, int row, int col, int disp, T invalid_
     return costAggr3;
 }
 
-template<typename T> 
+template<typename T>
 T aggregatedCostFromBottomRight4(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff4, T * min_disp4, T * pixel_4)
+    T * buff4, T * min_disp4, T * pixel_4, float current_class, float buff_class4, float & reset4)
 {
 
     /* ------------------*/
@@ -466,6 +534,8 @@ T aggregatedCostFromBottomRight4(T pixelCost, int row, int col, int disp, T inva
         if (disp == 0)
         {
             *min_disp4 = *std::min_element(&buff4[disp], &buff4[nb_disps]);
+            // if classes are different, reset history
+            reset4 = current_class == buff_class4;
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -473,15 +543,15 @@ T aggregatedCostFromBottomRight4(T pixelCost, int row, int col, int disp, T inva
         if(pixelCost != invalid_value)
         {
             //Previous cost
-            const uint8_t tmp1 = buff4[disp];
+            const T tmp1 = buff4[disp];
             //Previous cost at disparity-1
-            const uint8_t tmp2 = (disp>0) ? *pixel_4+P1 : std::numeric_limits<T>::max();
+            const T tmp2 = (disp>0) ? *pixel_4+P1 : std::numeric_limits<T>::max();
             //Previous cost at disparity+1
-            const uint8_t tmp3 = (disp<nb_disps-1) ? buff4[disp+1]+P1 : std::numeric_limits<T>::max();
+            const T tmp3 = (disp<nb_disps-1) ? buff4[disp+1]+P1 : std::numeric_limits<T>::max();
             //Minimum cost at previous point
-            const uint8_t tmp4 = (*min_disp4)+P2;
+            const T tmp4 = (*min_disp4)+P2;
             //Minimum path cost
-            costAggr4 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp4);
+            costAggr4 += reset4 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp4));
         }
     }
     //sava buffer data before writing new value in buffer
@@ -492,10 +562,10 @@ T aggregatedCostFromBottomRight4(T pixelCost, int row, int col, int disp, T inva
     return costAggr4;
 }
 
-template<typename T> 
+template<typename T>
 T aggregatedCostFromBottomRight5(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff5, T * min_disp5, T * pixel_5)
+    T * buff5, T * min_disp5, T * pixel_5, float current_class, float * buff_class5, float & reset5)
 {
 
     /* ------------------*/
@@ -511,6 +581,8 @@ T aggregatedCostFromBottomRight5(T pixelCost, int row, int col, int disp, T inva
         if (disp == 0)
         {
             *min_disp5 = *std::min_element(&buff5[disp+col*nb_disps], &buff5[nb_disps + col*nb_disps]);
+            // if classes are different, reset history
+            reset5 = current_class == buff_class5[col-direction.dcol];
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -526,7 +598,7 @@ T aggregatedCostFromBottomRight5(T pixelCost, int row, int col, int disp, T inva
             //Minimum cost at previous point
             const T tmp4 = (*min_disp5)+P2;
             //Minimum path cost
-            costAggr5 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp5);
+            costAggr5 += reset5 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp5));
         }
 
     }
@@ -541,7 +613,7 @@ T aggregatedCostFromBottomRight5(T pixelCost, int row, int col, int disp, T inva
 template<typename T>
 T aggregatedCostFromBottomRight6(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff6, T * buff_disp_6, T * min_disp6, T * pixel_6)
+    T * buff6, T * buff_disp_6, T * min_disp6, T * pixel_6, float current_class, float * buff_class6, float & reset6)
 {
 
     /* -----------------------------*/
@@ -558,6 +630,8 @@ T aggregatedCostFromBottomRight6(T pixelCost, int row, int col, int disp, T inva
         if (disp == 0)
         {
             *min_disp6 = *std::min_element(&buff_disp_6[disp], &buff_disp_6[nb_disps]);
+            // if classes are different, reset history
+            reset6 = current_class == buff_class6[col-direction.dcol];
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -573,7 +647,7 @@ T aggregatedCostFromBottomRight6(T pixelCost, int row, int col, int disp, T inva
             //Minimum cost at previous point
             const T tmp4 = (*min_disp6)+P2;
             //Minimum path cost
-            costAggr6 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp6);
+            costAggr6 += reset6 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp6));
         }
     }
     //sava buffer data before writing new value in buffer_disp
@@ -589,7 +663,7 @@ T aggregatedCostFromBottomRight6(T pixelCost, int row, int col, int disp, T inva
 template<typename T>
 T aggregatedCostFromBottomRight7(T pixelCost, int row, int col, int disp, T invalid_value,
     int nb_rows, int nb_cols, int nb_disps, T P1, T P2, Direction direction,
-    T * buff7, T * min_disp7)
+    T * buff7, T * min_disp7, float current_class, float * buff_class7, float &reset7)
 {
 
     /* ------------------------------*/
@@ -605,6 +679,8 @@ T aggregatedCostFromBottomRight7(T pixelCost, int row, int col, int disp, T inva
         if (disp == 0)
         {
             *min_disp7 = *std::min_element(&buff7[disp+(col-direction.dcol)*nb_disps], &buff7[nb_disps + (col-direction.dcol)*nb_disps]);
+            // if classes are different, reset history
+            reset7 = current_class == buff_class7[col-direction.dcol];
         }
         /*If pixelCost is equal to invalid value, aggregated cost must be equal to invalid value.
         So, it's useless to compute the minimum on tmp1,tmp2,tmp3,tmp4
@@ -620,7 +696,7 @@ T aggregatedCostFromBottomRight7(T pixelCost, int row, int col, int disp, T inva
             //Minimum cost at previous point
             const T tmp4 = (*min_disp7)+P2;
             //Minimum path cost
-            costAggr7 += std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp7);
+            costAggr7 += reset7 * (std::min({tmp1, tmp2, tmp3, tmp4}) - (*min_disp7));
         }
 
     }
@@ -647,7 +723,7 @@ void assignDirections(int* directions_in, Direction* dirs)
 }
 
 /* Explicitly instantiate all the templates needed to use libSGM as an external lib */
-template CostVolumes sgm(uint8_t * cv_in, uint8_t * p1_in, uint8_t * p2_in, int* directions_in,  int nb_rows,
- int nb_cols, int nb_disps, uint8_t invalid_value, bool cost_paths, bool overcounting);
-template CostVolumes sgm(float * cv_in, float * p1_in, float * p2_in, int* directions_in, int nb_rows, int nb_cols,
- int nb_disps, float invalid_value, bool cost_paths, bool overcounting);
\ No newline at end of file
+template CostVolumes sgm(uint8_t * cv_in, uint8_t * p1_in, uint8_t * p2_in, int* directions_in, unsigned long int nb_rows,
+unsigned long int nb_cols,unsigned int nb_disps, uint8_t invalid_value, float* segmentation, bool cost_paths, bool overcounting);
+template CostVolumes sgm(float * cv_in, float * p1_in, float * p2_in, int* directions_in, unsigned long int nb_rows,
+unsigned long int nb_cols, unsigned int nb_disps, float invalid_value, float* segmentation, bool cost_paths, bool overcounting);
\ No newline at end of file
